#region File Header
// /////////////////////////////////////////////////////////////
// File: AboutForm.cs	Class: Kennedy.Controls.ValidatingTextBox
// Date:10/21/2004		Author: Michael Kennedy
// Language: C#			Framework: .NET
//
// Copyright: Copyright (c) Michael Kennedy, 2004
// /////////////////////////////////////////////////////////////
#endregion

using System;
using System.ComponentModel;
using System.Text.RegularExpressions;

namespace Kennedy.Controls
{
	[System.ComponentModel.DefaultEvent("ValueChanged")]
	[System.ComponentModel.DefaultProperty("Text")]
	public abstract class ValidatingTextBox : System.Windows.Forms.TextBox
	{
		#region Validation Events
		
		[Category("Behavior")]
		[Description("Fires when the Text property is set to a value which is not valid " + 
				"against the current partial regular expression pattern.")]
		public event System.ComponentModel.CancelEventHandler ValidationFailed;

		[Category("Behavior")]
		[Description("Fires when the text changes to a valid value against the current " + 
				"partial regular expression pattern.")]
		public event System.EventHandler ValueChanged;

		#endregion

		#region Member Variables

		private Regex _regexPartial = null;
		private Regex _regexFull = null;
		private string _previousText = string.Empty;
		private string _partialPattern = string.Empty;
		private string _fullPattern = string.Empty;
		private bool _supressTextEvent = false;

		#endregion

		#region Construction

		public ValidatingTextBox()
		{
			//
			// Set the default pattern to simply require some 
			// non-white space content at the beginning of the text.
			//
			this.PatternPartial = @"^\S+";
			this.PatternFull = @"^\S+";

			this.TextChanged += new EventHandler(ValidatingTextBox_TextChanged);
		}

		#endregion

		#region Validation Related Properties

		protected virtual string PatternFull
		{
			get
			{
				lock (this)
				{
					return _fullPattern;
				}
			}
			set
			{
				lock (this)
				{
					if (value == null || value.Length == 0)
					{
						throw new ArgumentException("Pattern cannot be a zero length string.", "PatternFull");
					}

					_fullPattern = value;
					_regexFull = new Regex(_fullPattern, RegexOptions.Compiled);
				}
			}
		}

		protected virtual string PatternPartial
		{
			get
			{
				lock (this)
				{
					return _partialPattern;
				}
			}
			set
			{
				lock (this)
				{
					if (value == null || value.Length == 0)
					{
						throw new ArgumentException("Pattern cannot be a zero length string.", "PatternPartial");
					}

					_partialPattern = value;
					_regexPartial = new Regex(_partialPattern, RegexOptions.Compiled);
				}
			}
		}

		[Description("Indicates whether the current text is valid when consider the final input.")]
		[Category("Behavior")]
		public virtual bool IsValid
		{
			get
			{
				lock (this)
				{
					return _regexFull.IsMatch(this.Text);
				}
			}
		}

		[Description("Indicates whether the current text is valid when consider the partial, incomplete input.")]
		[Category("Behavior")]
		public virtual bool IsValidPartial
		{
			get
			{
				lock (this)
				{
					return _regexPartial.IsMatch(this.Text);
				}
			}
		}

		#endregion

		#region Validation Related Methods

		public virtual bool ValidateString(string text, bool useFinal)
		{
			lock (this)
			{
				Regex regex = useFinal ? _regexFull : _regexPartial;
				return regex.IsMatch(text);
			}
		}

		private void ValidatingTextBox_TextChanged(object sender, EventArgs e)
		{
			lock (this)
			{
				if (_supressTextEvent)
				{
					return;
				}

				string undoValue = _previousText;
				_previousText = this.Text;

				if (this.IsValidPartial)
				{
					Fire_ValueChanged();
					return;
				}

				if (Fire_ValidationFailed())
				{
					int index = this.SelectionStart;

					_previousText = undoValue;
					SetTextWithoutEvents(undoValue);

					int selectPos = Math.Max(0, Math.Min(index - 1, undoValue.Length));
					Select(selectPos, 0);
				}
			}
		}

		protected void SetTextWithoutEvents(string text)
		{
			lock (this)
			{
				bool eventsAlreadySuppressed = _supressTextEvent;

				_supressTextEvent = true;

				this.Text = text;

				if (!eventsAlreadySuppressed)
				{
					_supressTextEvent = false;
				}
			}
		}

		public void RevalidateText()
		{
			ValidatingTextBox_TextChanged(this, EventArgs.Empty);
		}

		#endregion

		#region Validation Event Management Code

		private bool Fire_ValidationFailed()
		{
			lock (this)
			{
				if (ValidationFailed == null)
				{
					return true;
				}

				System.ComponentModel.CancelEventArgs e = new System.ComponentModel.CancelEventArgs(true);
				ValidationFailed(this, e);

				return e.Cancel;
			}
		}

		private void Fire_ValueChanged()
		{
			lock (this)
			{
				if (ValueChanged == null)
				{
					return;
				}

				ValueChanged(this, EventArgs.Empty);
			}
		}

		#endregion

	}
}
